# SDT_Graphs.R
# 
# This R script file creates various signal detection graphs using the *_grf.txt
# files created by RscorePlus. The snippets below are keyed to the examples in
# the RscorePlus User's Manual
# 
# Lewis O. Harvey, Jr. Department of Psychology and Neuroscience University of
# Colorado Boulder, Colorado, USA 26 September 2013
# 
# When this whole script is executed it first prompts you for the grf file
# produced by RscorePlus. It then defines eight different plot functions (plot1
# ... plot8). Finally it executes these eight plot functions to draw the eight 
# graphs in one window. You can then plot any of the individual graphs by giving
# the appropriate plot command.
# 
# To create any graph from the graphical output of RscorePlus there are three
# steps to follow:
# 
# 1. Read the contents of the file into an R data frame;
# 2. Select the subset of data to plot, typically a single data set;
# 3. Issue plot commands using the appropriate pairs of variables.
# 
# For multiple data sets from one RscoreRun, this script loops through the data
# sets one at a time using variable sn (set number). 
# Hitting return in the console window advances to the next data set.

#######################################################################
# Define the eight plot functions
# The plots are implimented as functions so that they can be
# individually plotted (e.g., plot1(set_number, sig0, sig1))
# The set number allows you to plot a sigle data set in a graphics file
# that contains more than one RscorePlus analysis. Some of the plot
# commands additionally allow you to plot single pairs of signal
# conditons (e.g., plot1() and plot2() that draw ROC functions)

# Plot 1
#######################################################################
# Observed vs. Predicted ROC (probability coordinates)
# sn -> data set number
# indx -> for plotting a single ROC (x-axis)
# indy -> for plotting a single ROC (y-axis)
plot1 <- function(data.set.number = 1, signal.0 = -1, signal.1 = -1) {
	sn <- data.set.number
    indx <- signal.0
    indy <- signal.1
    ndata.sets <- length(levels(as.factor(df.all$data_set)))
    
    # check to see if more than one ROC should be plotted
	many.plots = FALSE
	if (nsig > 2 && indx == -1 && indy == -1) many.plots = TRUE
	if(indx == -1 || indx > nsig - 1) indx <- 0
	if(indy == -1 || indy > nsig - 1) indy <- 1
	
	# make sure sn is within range in data frame
	if(sn < 0 || sn > ndata.sets) sn <- 1

	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	
	ind0 <- match("Obs_pYs0",  names(df))	# find s0 observed p(Y|s) column
	ind1 <- match("Pred_pYs0", names(df))	# find s0 predicted p(Y|s) column

	# plot s0-s1 ROC
	par(pty="s")								# make plot square
    ob0 <- ind0 + indx
    ob1 <- ind0 + indy
	plot(df[, ob1] ~ df[, ob0],
        type = "p",
        pch = 19,
		xlim = c(0, 1),
		ylim = c(0, 1),
		xlab = "False Alarm Rate (Probability)",
		ylab = "Hit Rate (Probability)",
		main = paste("Data Set ", sn, "\n", "Observed and Predicted ROC\n", model.type, " model"))
    pr0 <- ind1 + indx
    pr1 <- ind1 + indy
	lines(df[, pr0], df[, pr1])
	abline(0, 1, col = "gray", lty = "solid")
	
	# plot the remaining ROCs relative to s0
	colors <- rainbow(nsig - 1)
	if (many.plots){
		for (i in 2:(nsig-1)){
            ob0 <- ind0 + indx
            ob1 <- ind0 + indx + i
			points(df[, ob1] ~ df[, ob0],
                type = "p",
				pch = 19,
                col = colors[i])
            pr0 <- ind1 + indx
            pr1 <- ind1 + indx + i
			lines(df[, pr0], df[, pr1], col = colors[i])
		}
	}
	par(pty="m")
}

# Plot 2
#######################################################################
# Observed vs. Predicted ROC (z-score probability coordinates)
# sn -> data set number
# indx -> for plotting a single ROC (x-axis)
# indy -> for plotting a single ROC (y-axis)
plot2 <- function(data.set.number = 1, signal.0 = -1, signal.1 = -1) {
	sn <- data.set.number
    indx <- signal.0
    indy <- signal.1
    ndata.sets <- length(levels(as.factor(df.all$data_set)))
    
    # check to see if more than one ROC should be plotted
	many.plots = FALSE
	if (nsig > 2 && indx == -1 && indy == -1) many.plots = TRUE
	if(indx == -1 || indx > nsig - 1) indx <- 0
	if(indy == -1 || indy > nsig - 1) indy <- 1
	
	# make sure sn is within range in data frame
	if(sn < 0 || sn > ndata.sets) sn <- 1

	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
    
	ind0 <- match("Obs_zYs0", names(df))	# find s0 observed z(Y|s) column
	ind1 <- match("Pred_zYs0", names(df))	# find s0 predicted z(Y|s) column
	lim <- 1.1 * max(abs(df[, ind0:(ind0+nsig-1)]), na.rm = TRUE)
    
	# plot s0-s1 ROC
	par(pty="s")	# make square
	ob0 <- ind0 + indx
	ob1 <- ind0 + indy
	plot(df[, ob1] ~ df[, ob0],
        type = "p", pch = 19,
		xlim = c(-lim, lim),
		ylim = c(-lim, lim),
		xlab = "False Alarm Rate (z-score)",
		ylab = "Hit Rate (z-score)",
		main = paste("Data Set ", sn, "\n", "Observed and Predicted z-score ROC\n", model.type, " model"))
    pr0 <- ind1 + indx
    pr1 <- ind1 + indy
	lines(df[, pr0], df[, pr1])
	abline(0, 1, col = "gray", lty = "solid")
	
	# plot the ROCs (s2...)
	colors <- rainbow(nsig - 1)
	if (many.plots){
	    for (i in 2:(nsig-1)){
	        ob0 <- ind0 + indx
	        ob1 <- ind0 + indx + i
	        points(df[, ob1] ~ df[, ob0],
	               type = "p",
	               pch = 19,
	               col = colors[i])
	        pr0 <- ind1 + indx
	        pr1 <- ind1 + indx + i
	        lines(df[, pr0], df[, pr1], col = colors[i])
	    }
	}
	par(pty="m")
}

# Plot 3
#######################################################################
# Observed vs. Predicted Response Frequencies
plot3 <- function(data.set.number = 1) {
	sn <- data.set.number
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	ind0 <- match("Obs_Freq0",  names(df))	# find observed s0 response frequency column
	ind1 <- match("Pred_Freq0", names(df))	# find predicted s0 response frequency column
	max.freq <- 1.2 * max(df[, ind0:ind1], na.rm = TRUE)
	par(pty="s")	# make square

	# plot s0 response frequencies
	plot(df[, ind0], df[, ind0 + nsig], type = "p", pch = 19,
		xlim = c(0, max.freq),
		ylim = c(0, max.freq),
		xlab = "Observed Rating Frequencies",
		ylab = "Predicted Rating Frequencies",
		main = paste("Data Set ", sn, "\n", "Observed vs. Predicted Response Frequencies\n", model.type, " model"))
	
	# plot the remaining response frequencies (s1...)
	colors <- rainbow(nsig - 1)
	for (i in (ind0+1):(ind0+nsig-1)){
		points(df[, i], df[, i+nsig], type = "p",
			pch = 19, col = colors)
	}
	abline(0, 1, col = "black", lty = "solid")
	par(pty="m")
}

# Plot 4
#######################################################################
# Observed vs. Predicted Response Probabilities
plot4 <- function(data.set.number = 1) {
	sn <- data.set.number
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	ind0 <- match("Obs_Prob0", names(df))	# find observed s0 response probability column
	ind1 <- match("Pred_Prob0", names(df))	# find predicted s0 response probability column
	max.prob <- 1.2 * max(df[, ind0:ind1], na.rm = TRUE)
	par(pty="s")	# make square

	# plot s0 response probabilities
	plot(df[, ind0], df[, ind0 + nsig], type = "p", pch = 19,
		xlim = c(0, max.prob),
		ylim = c(0, max.prob),
		xlab = "Observed Rating Probabilities",
		ylab = "Predicted Rating Probailities",
		main = paste("Data Set ", sn, "\n", "Observed vs. Predicted Response Probabilities\n", model.type, " model"))
	
	# plot the remaining response probabiliites (s1...)
	colors <- rainbow(nsig - 1)
	for (i in (ind0+1):(ind0+nsig-1)){
		points(df[, i], df[, i+nsig], type = "p",
			pch = 19, col = colors)
	}
	abline(0, 1, col = "black", lty = "solid")
	par(pty="m")
}

# Plot 5
#######################################################################
# Observed and Predicted critical operating characteristic
plot5 <- function(data.set.number = 1) {
	sn <- data.set.number
	df <- subset(df.all, data_set == sn)		# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	ind0 <- match("Obs_pYs1", names(df))		# find the observed hit rate
	ind1 <- match("Obs_pHitXc1", names(df))		# find the observed posterior of hit
	ind2 <- match("Pred_pYs1", names(df))		# find the predicted hit rate
	ind3 <- match("Pred_pHitXc1", names(df))	# find the predicted posterior of hit

	par(pty="s")	# make square

	# plot s1 posterior
	plot(df[, ind1] ~ df[, ind0], 
		type = "p", pch = 19,
		xlim = c(0, 1),
		ylim = c(0, 1),
		xlab = "Hit Rate (Probability)",
		ylab = "Posterior Probability of a Hit",
		main = paste("Data Set ", sn, "\n", "Critical Operating Characteristic\n", model.type, " model"))
	lines(df[, ind3] ~ df[, ind2])
	lines(c(0, 1), c(0.9, 0.9), col = "gray")
	text(0.8, 0.92, "Beyond Reasonable Doubt", cex = 0.7)

	inter <- approx(df[, ind2], df[, ind3], n=1000)
	ind4 <- match(TRUE, inter$y < 0.9)
	if (!is.na(ind4)) {
		xc <- inter$x[ind4]
		yc <- inter$y[ind4]
		lines(c(xc, xc), c(0, yc), col = "gray")
		text(xc-.025, .2, "Critical Hit Rate", cex = 0.7, srt = 90)
	}
	
	# plot the remaining posterior probabilities (s2...)
	colors <- rainbow(nsig - 1)
	if (nsig > 2){
		for (i in 1:(nsig-1)){
			points(df[, ind0+i], df[, ind1+i], type = "p",
				pch = 19, col = colors[i])
			lines(df[, ind2+i], df[, ind3+i], col = colors[i])
			inter <- approx(df[, ind2+i], df[, ind3+i], n=1000)
			ind4 <- match(TRUE, inter$y < 0.9)
			if (!is.na(ind4)) {
				xc <- inter$x[ind4]
				yc <- inter$y[ind4]
				lines(c(xc, xc), c(0, yc), col = colors[i])
			}
		}
	}
par(pty="m")
}

# Plot 6
#######################################################################
# plot residuals of observed probabilites and predicted probabilities
plot6 <- function(data.set.number = 1) {
	sn <- data.set.number
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	ndata <- nrat * nsig
		
	ind0 <- match("Obs_Prob0", names(df))		# find observed freqs
	ind1 <- match("Pred_Prob0", names(df))		# find predicted freqs
	
	obs <- df[!is.na(df[,ind0]), ind0]
	pre <- df[!is.na(df[,ind1]), ind1]
	resid <- obs - pre
	
	for(i in 2:nsig-1)
	{
		obs <- df[!is.na(df[,ind0+i]), ind0+i]
		pre <- df[!is.na(df[,ind1+i]), ind1+i]
		resid <- c(resid, (obs - pre))
	}
	ymax <- max(resid)
	ymin <- min(resid)
	yymx <- max(abs(ymax), abs(ymin))
	yymx <- yymx * 1.5
	plot(resid,
		pch = 19, col = "black", type='n',
		ylim = c(-yymx, yymx),
		xlab = "Rating",
		ylab = "Residual",
		main = paste("Data Set ", sn, "\n", "Observed Minus Predicted Rating Probabilities"),
		xlim = c(1, ndata))
		
	points(resid[1:nrat], pch = 19, col = "black")
	colors <- rainbow(nsig - 1)
	for (i in 1:(nsig-1))
	{
		i0 <- (nrat * i) + 1
		i1 <- i0 + nrat
		points(i0:i1, resid[i0:i1], pch = 19, col=colors[i])
	}
	
	for (i in 1:nsig-1)
	{
		i0 <- (nrat * i) + 0.5
		abline(v=i0, col="gray")
	}
	abline(h=0.0, col="gray")
}

# Plot 7
#######################################################################
# Observed and Predicted posterior probability of signal given response
plot7 <- function(data.set.number = 1) {
	sn <- data.set.number
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	ind0 <- match("Rating_Category", names(df))	# find the rating scale
	ind1 <- match("Obs_pSr1", names(df))		# find posterior prob of signal 1
	ind2 <- match("Pred_pSr1", names(df))		# find posterior prob of signal 1

	par(pty="m")	# don't make square

	# plot s1 posterior
	plot(df[, ind0], df[, ind1], type = "p", pch = 19,
		ylim = c(0, 1),
		xlab = "Rating Categories",
		ylab = "Posterior Probability of Signal, P(s|R)",
		main = paste("Data Set ", sn, "\n", "Posterior Probability of Signal\n", model.type, " model"))
	lines(df[, ind0], df[, ind2])
	
	# plot the remaining posterior probabilities (s2...)
	colors <- rainbow(nsig - 1)
	if (nsig > 2){
		for (i in 1:(nsig-1)){
			points(df[, ind0], df[, ind1+i], type = "p",
				pch = 19, col = colors[i])
			lines(df[, ind0], df[, ind1+i], col = colors[i])
		}
	}
	par(pty="m")
}

# Plot 8
#######################################################################
# Signal Detection Model with Decision Criteria
plot8 <- function(data.set.number = 1) {
	sn <- data.set.number
    ndata.sets <- length(levels(as.factor(df.all$data_set)))
    
	# make sure sn is within range in data frame
	if(sn < 0 || sn > ndata.sets) sn <- 1
	
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	par(pty="m")

	# find the maximum density value for setting y-axis
	ind0 <- match("Pred_Density0", names(df))	# find s0 density column
	ind1 <- ind0 + nsig - 1
	max.density <- 1.2 * max(df[, ind0:ind1], na.rm = TRUE)
	min.density <- -0.05
	rng.density <- max.density - min.density
	max.criterion <- max(df$Xc, na.rm = TRUE)

	# plot s0 probability density function
	plot(df[, ind0] ~ df$Xc, type = "l", col = "black",
		xlim = c(min(df$Xc, na.rm = TRUE), max.criterion),
		ylim = c(min.density, max.density),
		xlab = "Decision Axis",
		ylab = "Probability Density",
		main = paste("Data Set ", sn, "\n", "Signal Detection Model with Decision Criteria\n", model.type, " model"))
#		main = "Blake and Malissa")
	
	# plot the remaining probability density functions (s1...)
	colors <- rainbow(nsig-1)
	for (i in 1:(nsig-1)){
		lines(df[, ind0+i] ~ Xc, data = df, type = "l", col = colors[i])
	}
	abline(h = 0, col = "gray", lty = "solid")

	# extract the criterion coordinates
	crit <- subset(df, Criteria != is.na(Criteria))[c("Xc", "Xc_se", "Criteria")]
	xc <- df$Xc[1:ncrt]	
	# plot the decision criterion markers
	for (i in 1:ncrt){
		segments(crit$Xc[(2*i)-1], crit$Criteria[(2*i)-1], crit$Xc[2*i], crit$Criteria[2*i], col="red")
	}
# plot the standard error of each criterion marker
	for (i in 1:ncrt){
		segments(crit$Xc[2*i]-crit$Xc_se[2*i], crit$Criteria[2*i],
		crit$Xc[2*i]+crit$Xc_se[2*i], crit$Criteria[2*i], col = "red")
	}
	# label the response categories
	xc.lab <- rep(0, nrat)
	xc.lab[2:ncrt] <- xc[1:ncrt-1] + diff(xc)/2
	xc.lab[1] <- xc[1] - (diff(xc)/2)[1]
	xc.lab[nrat] <- xc[ncrt] + (diff(xc)/2)[ncrt-1]
	text(xc.lab, -rng.density * 0.05, labels = 1:nrat, cex = 0.7)
	par(pty="m")
}

#######################################################################
# make all eight plots in one window
plot.all <- function(sn) {
	par (mfcol = c(4 ,2))
	plot1(sn)
	plot2(sn)
	plot3(sn)
	plot4(sn)
	plot5(sn)
	plot6(sn)
	plot7(sn)
	plot8(sn)
	par (mfcol = c(1 ,1))
}


# main program loop
#######################################################################
#######################################################################

# Read the RscorePlus *_grf.txt file into a data frame
fn <- file.choose()							# choose the file to read
df.all <- read.delim(fn, header = TRUE)		# reads tab delimited file
totalNumberOfDataSets <- length(levels(factor(df.all$data_set)))

for (sn in 1:totalNumberOfDataSets)
{
	df <- subset(df.all, data_set == sn)	# select a data set
	nsig <- df$nsig[1]						# number of signals
	nrat <- df$nrat[1]						# number of ratings
	ncrt <- nrat - 1						# number of criteria
	model.type <- df$Model[1]				# get the model type
	plot.all(sn)
	if (totalNumberOfDataSets > 1) par(ask = TRUE)
	if (sn == totalNumberOfDataSets) par(ask = FALSE)
}
